# -*- coding: utf-8 -*-
""" Python package tests """

import re

import rpm

from FedoraReview import CheckBase, RegistryBase


class Registry(RegistryBase):
    """Register all checks in this file in group 'Python'"""

    group = "Python"

    def is_applicable(self):
        """Return true if this is a python package."""
        if self.is_user_enabled():
            return self.user_enabled_value()
        return self.checks.spec.name.startswith("python") or self.checks.rpms.find(
            "*.pyc"
        )


class PythonCheckBase(CheckBase):
    """Base class for all python  checks."""

    def __init__(self, base):
        CheckBase.__init__(self, base, __file__)


class CheckPythonBuildRequires(PythonCheckBase):
    """Check if the BuildRequires have the mandatory elements."""

    def __init__(self, checks):
        PythonCheckBase.__init__(self, checks)
        self.url = (
            "https://docs.fedoraproject.org/en-US"
            "/packaging-guidelines/Python/#_dependencies"
        )
        self.text = "Package contains BR: python2-devel or python3-devel"
        self.automatic = True

    def run_on_applicable(self):
        br = self.spec.build_requires
        passed = "python2-devel" in br or "python3-devel" in br
        self.set_passed(self.PASS if passed else self.FAIL)


class CheckPythonRequires(PythonCheckBase):
    """Check that there is no unversioned Python dependencies"""

    def __init__(self, base):
        PythonCheckBase.__init__(self, base)
        self.url = (
            "https://docs.fedoraproject.org/en-US"
            "/packaging-guidelines/Python/#_dependencies"
        )
        self.text = (
            "Packages MUST NOT have dependencies (either build-time "
            "or runtime) on packages named with the unversioned python- "
            "prefix unless no properly versioned package exists. "
            "Dependencies on Python packages instead MUST use names "
            "beginning with python2- or python3- as appropriate."
        )
        self.automatic = True
        self.type = "MUST"

    def run_on_applicable(self):
        """Run the check"""
        regex = re.compile("(^python-)|(-python$)|(-python-)", re.M)
        br = self.spec.build_requires
        rr = self.spec.get_requires()

        if regex.search("\n".join(br)) or regex.search("\n".join(rr)):
            self.set_passed(self.FAIL, "Unversionned Python dependency found.")
            return
        self.set_passed(self.PASS)


class CheckPythonX_Site(PythonCheckBase):
    """
    MUST: One must not use any of the following in the %files section:

    %{python3_sitelib}/*
    %{python3_sitearch}/*
    %{python2_sitelib}/*
    %{python2_sitearch}/*

    https://pagure.io/packaging-committee/issue/782

    """

    def __init__(self, base):
        PythonCheckBase.__init__(self, base)
        self.url = "https://pagure.io/packaging-committee/issue/782"
        self.text = (
            "Python packages must not contain %{pythonX_site(lib|arch)}/* in %files"
        )
        self.automatic = True
        self.type = "MUST"

    def run_on_applicable(self):
        has_python_site = False
        python_sitelib = re.compile(
            rpm.expandMacro(r"^%{python2_sitelib}\/\*$")
            + "|"
            + rpm.expandMacro(r"^%{python2_sitearch}\/\*$")
            + "|"
            + rpm.expandMacro(r"^%{python3_sitelib}\/\*$")
            + "|"
            + rpm.expandMacro(r"^%{python3_sitearch}\/\*$")
        )

        for pkg in self.spec.packages:
            for line in self.spec.get_files(pkg):
                if python_sitelib.match(line):
                    has_python_site = True
                    break
        if has_python_site:
            self.set_passed(
                self.FAIL, "Package contains %{pythonX_site(lib|arch)}/* in %files"
            )
        else:
            self.set_passed(self.PASS)
